type Merge<T extends object> = Partial<{
    [K in keyof T]: T[K] extends object ? Merge<T[K]> : T[K];
}>;
declare function merge<T extends object>(a: Merge<T>, b: Merge<T>): void;
declare const is: {
    arr: (arg: any) => arg is any[];
    bol: (a: unknown) => a is boolean;
    str: (a: unknown) => a is string;
    num: (a: unknown) => a is number;
    fun: (a: unknown) => a is Function;
    und: (a: unknown) => a is undefined;
    nul: (a: unknown) => a is null;
    set: (a: unknown) => a is Set<unknown>;
    map: (a: unknown) => a is Map<unknown, unknown>;
    obj: (a: unknown) => a is object;
    nan: (a: unknown) => a is number;
};
declare const isServer: () => boolean;
/**
 * each
 */
type EachFn<Value, Key, This> = (this: This, value: Value, key: Key) => void;
type Eachable<Value = any, Key = any, This = any> = {
    forEach(cb: EachFn<Value, Key, This>, ctx?: This): void;
};
declare const each: <Value, Key, This>(obj: Eachable<Value, Key, This>, fn: EachFn<Value, Key, This>) => void;
declare const flush: <Value extends Function, Key, This>(obj: Eachable<Value, Key, This>, ...args: any[]) => void;
/**
 * math
 */
declare const sig: (value?: number, digit?: number) => number;
declare const dig: (x?: number) => number;
declare const fig: (x?: number) => number;
declare const ext: (src?: string) => string;
declare const vec2: (x?: number, y?: number, out?: any[] | Float32Array<ArrayBuffer>) => Vec2;
type Vec2 = [x: number, y: number];
declare const addV: (a: Vec2, b: Vec2, out?: Vec2) => Vec2;
declare const subV: (a: Vec2, b: Vec2, out?: Vec2) => Vec2;
declare const cpV: (a: Vec2, out?: Vec2) => Vec2;
/**
 * fullscreen
 */
declare const fullscreen: (target: HTMLElement) => void;
declare const replace: (char?: string, from?: string, to?: string) => string;
declare const mark: (k: string) => void;

type HTMLMap = HTMLElementTagNameMap;
type HTMLTag = keyof HTMLMap;
type HTMLNode<T extends HTMLTag = HTMLTag> = HTMLMap[T] | string | number | null | undefined;
type Component<T extends HTMLTag, P = {}, Child = HTMLMap[T]> = (props: P) => Child;
type Props$1<T extends HTMLTag> = Merge<HTMLMap[T]> & {
    key?: string;
    ref?: (el: HTMLMap[T]) => void;
    children?: HTMLNode | HTMLNode[];
};
declare function append<El extends Node>(child: Node | string | number | null | undefined, el: El): void;
declare function remove<El extends Node>(child: Node, el: El): void;
declare function create<T extends HTMLTag>(type: T, props?: Props$1<T>, ...args: HTMLNode[]): HTMLMap[T];
declare function create<T extends HTMLTag, P = {}, Child = HTMLMap[T]>(type: Component<T, P, Child>, props?: P, ...args: HTMLNode[]): Child;

/**
 * Vector
 */
interface XYZVector {
    x?: number;
    y?: number;
    z?: number;
    w?: number;
}
declare const isVector: (a: object) => a is XYZVector;
declare const isEuler: (a: object) => a is XYZVector;
declare const isQuaternion: (a: object) => a is XYZVector;
/**
 * Matrix
 */
interface Matrix {
    elements: number[];
}
declare const isMatrix: (a: object) => a is Matrix;
/**
 * Color
 */
interface RGBColor {
    r?: number;
    g?: number;
    b?: number;
    a?: number;
}
declare const isHex: (a: string) => boolean;
declare const isColor: (a: object) => a is RGBColor;
interface SelectOptions {
    options: string[];
}
declare const isSelect: (a: unknown) => a is SelectOptions;
/**
 * Button
 */
interface ButtonOnClick {
    onclick: () => void;
}
declare const isButton: (a: unknown) => a is ButtonOnClick;
interface FilesSource {
    src: string;
    name?: string;
    size?: number;
    type?: string;
}
declare const isFiles: (a: unknown) => a is FilesSource;
interface AudioSource extends FilesSource {
}
declare const isAudio: (a: unknown) => a is AudioSource;
interface ImageSource extends FilesSource {
    alt?: string;
}
declare const isImage: (a: unknown) => a is ImageSource;
interface VideoSource extends FilesSource {
    alt?: string;
}
declare const isVideo: (a: unknown) => a is AudioSource;
/**
 * Uniform
 */
type Uniform<T> = {
    value: T;
};
declare const isU: <T>(a: unknown) => a is Uniform<T>;
/**
 * Input
 */
type Value<T> = Uniform<T> | T;
type Input = Value<boolean> | Value<string> | Value<number> | Value<number[]> | Value<[x: number]> | Value<[x: number, y: number]> | Value<[x: number, y: number, z: number]> | Value<[x: number, y: number, z: number, w: number]> | Value<XYZVector> | Value<RGBColor>;
interface Target {
    [key: string]: Input | Target | any;
}
type Callback$1<T extends Target = Target> = (key: keyof T & string, arg: T[keyof T & string]) => void;
interface Ctrl<T extends Target = Target> {
    isC: true;
    get parent(): null | Ctrl;
    set parent(parent: Ctrl);
    get id(): string;
    set id(id: string);
    get current(): T;
    writes: Set<Callback$1<T>>;
    events: Set<Callback$1<T>>;
    actors: Set<Function>;
    mounts: Set<Function>;
    cleans: Set<Function>;
    mount(): void;
    clean(): void;
    sub(fn?: () => void): () => void;
    get(): number;
    act(): void;
    set: Callback$1<T>;
    run: Callback$1<T>;
    ref(target: T | null): void;
    cache: any;
}
declare const isC: <T extends Target>(a: unknown) => a is Ctrl<T>;
interface Attach<Arg, T extends Target = Target, K extends keyof T & string = keyof T & string> {
    children?: string;
    a: Arg & T[K];
    c: Ctrl<T>;
    k: K;
}
interface CustomPlugin<Arg = unknown> {
    is(a: unknown): a is Arg;
    el(props: Attach<Arg>): any;
}

declare const store: Set<Ctrl<Target>>;
declare function ctrl<T extends Target>(current: T, id?: string): Ctrl<T>;
declare namespace ctrl {
    var create: typeof create;
    var append: typeof append;
    var remove: typeof remove;
    var plugin: CustomPlugin<unknown>[];
    var use: (...args: CustomPlugin[]) => number;
    var pluginParent: Node | null;
    var layersParent: Node | null;
    var mounts: Set<Function>;
}
declare const register: (override: any) => void;

declare const SUPPORT: Record<string, boolean>;
/**
 * https://github.com/pmndrs/use-gesture/blob/main/packages/core/src/config/dragConfigResolver.ts
 */
declare const getDevice: (lock?: boolean) => "mouse" | "touch" | "pointer";
declare const getClientVec2: (e: any, device: unknown, out: Vec2) => Vec2;
declare const EVENT_FOR_DRAG: any;
declare const dragEvent: (callback: Callback) => {
    _active: boolean;
    active: boolean;
    device: string;
    _value: Vec2;
    value: Vec2;
    delta: Vec2;
    offset: Vec2;
    movement: Vec2;
    target: Element | null;
    event: Event | null;
    memo: any;
    isDragStart: boolean;
    isDragging: boolean;
    isDragEnd: boolean;
    onDrag: () => void;
    onDragStart: (e: Event) => void;
    onDragging: (e: Event) => void;
    onDragEnd: (e: Event) => void;
    onMount: (target: Element) => void;
    onClean: () => void;
    ref: (el: Element) => void;
};
type Drag = ReturnType<typeof dragEvent>;
type Callback = (self: Drag) => void;

interface Config {
    token?: string;
    appId?: string;
    secret?: string;
    roomName?: string;
}

type PublishData<T extends object, K extends keyof T & string = keyof T & string> = [key: K, value: T[K]];
declare function sync<T extends object = object>(target: T, id?: string): {
    events: Set<Callback$1<T>>;
    writes: Set<Callback$1<T>>;
    actors: Set<Function>;
    mounts: Set<Function>;
    cleans: Set<Function>;
    mount: () => Promise<void>;
    clean: () => void;
    sub: (fn?: (_k: keyof T & string, _v: T[keyof T & string]) => void) => () => void;
    act: () => void;
    run: (k: keyof T & string, a: T[keyof T & string]) => void;
    set: (data: PublishData<T, keyof T & string>) => Promise<void>;
    get(): number;
    readonly joined: {
        room: any;
        member: any;
        data: any;
    };
    readonly current: T;
    isC: boolean;
    config: Config;
};
type Sync = ReturnType<typeof sync>;

declare const indexed: <T extends object = object>(id?: string) => {
    open: () => Promise<unknown>;
    drop: () => void;
    transact: (mode?: string) => Promise<any>;
    all: () => Promise<unknown>;
    one: (key: keyof T & string) => Promise<T[keyof T & string] | undefined>;
    set: (key: keyof T & string, value: T[keyof T & string]) => Promise<unknown>;
    del: (key: keyof T & string) => Promise<unknown>;
};
declare function save<T extends object = object>(target: T, id?: string): {
    events: Set<Callback$1<T>>;
    writes: Set<Callback$1<T>>;
    actors: Set<Function>;
    mounts: Set<Function>;
    cleans: Set<Function>;
    mount: () => Promise<void>;
    clean: () => void;
    sub: (fn?: () => void) => () => void;
    act: () => void;
    run: (k: keyof T & string, a: T[keyof T & string]) => void;
    set: (k: keyof T & string, a: T[keyof T & string]) => void;
    get(): number;
    readonly current: T;
    idb: {
        open: () => Promise<unknown>;
        drop: () => void;
        transact: (mode?: string) => Promise<any>;
        all: () => Promise<unknown>;
        one: (key: keyof T & string) => Promise<T[keyof T & string] | undefined>;
        set: (key: keyof T & string, value: T[keyof T & string]) => Promise<unknown>;
        del: (key: keyof T & string) => Promise<unknown>;
    };
    isC: boolean;
};
type Save = ReturnType<typeof save>;

declare const wheelEvent: (callback: WheelCallback) => {
    active: boolean;
    _active: boolean;
    _value: Vec2;
    value: Vec2;
    delta: Vec2;
    offset: Vec2;
    movement: Vec2;
    target: Element;
    event: Event;
    timeout: number;
    clearTimeout: () => void;
    memo: {};
    isWheelStart: boolean;
    isWheeling: boolean;
    isWheelEnd: boolean;
    onWheel: () => void;
    onWheelStart: (e: WheelEvent) => void;
    onWheeling: (e: Event) => void;
    onWheelEnd: (e: Event) => void;
    onMount: (target: Element) => void;
    onClean: () => void;
    ref: (el: Element | null) => void;
};
type WheelState = ReturnType<typeof wheelEvent>;
type WheelCallback = (state: WheelState) => void;

interface Props<T extends Target> {
    c: Ctrl<T>;
}
declare function PluginItem<T extends Target>(props: Props<T>): HTMLDivElement;
declare function DefaultPlugin<T extends Target>(props: Attach<unknown, T>): any;

export { type SelectOptions as $, mark as A, sync as B, type Component as C, type Sync as D, EVENT_FOR_DRAG as E, indexed as F, save as G, type HTMLMap as H, type Save as I, wheelEvent as J, type WheelCallback as K, PluginItem as L, type Merge as M, DefaultPlugin as N, isVector as O, type Props$1 as P, isEuler as Q, isQuaternion as R, SUPPORT as S, type Matrix as T, isMatrix as U, type Vec2 as V, type WheelState as W, type XYZVector as X, type RGBColor as Y, isHex as Z, isColor as _, getClientVec2 as a, isSelect as a0, type ButtonOnClick as a1, isButton as a2, type FilesSource as a3, isFiles as a4, type AudioSource as a5, isAudio as a6, type ImageSource as a7, isImage as a8, type VideoSource as a9, isVideo as aa, type Uniform as ab, isU as ac, type Value as ad, type Input as ae, type Target as af, type Callback$1 as ag, type Ctrl as ah, isC as ai, type Attach as aj, type CustomPlugin as ak, create as b, ctrl as c, dragEvent as d, type HTMLTag as e, type HTMLNode as f, getDevice as g, append as h, remove as i, is as j, isServer as k, each as l, merge as m, flush as n, sig as o, dig as p, fig as q, register as r, store as s, ext as t, addV as u, vec2 as v, subV as w, cpV as x, fullscreen as y, replace as z };
